<?php
namespace Emblz\SendTrackNumbers\Helper;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\App\Helper\Context;
use \Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Shipment\Track;
use \Magento\Sales\Model\Order\Shipment;
use \Magento\Catalog\Model\ProductFactory;
use \Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\ResourceModel\Order\Shipment\Track\Collection as TrackCollection;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    const XML_PATH_SHIPMENT_AUTH_TOKEN = 'shipping/shipment_email_settings_send_track/auth_token';
    const XML_PATH_SHIPMENT_SEND_TRACK_URL = 'shipping/shipment_email_settings/track_send_api_url';
    const XML_PATH_SHIPMENT_RPS = 'shipping/shipment_email_settings/rps';
    const XML_PATH_SHIPMENT_ATTRS_TO_SEND = 'shipping/shipment_email_settings_send_track/customer_attributes';
    const XML_PATH_SHIPMENT_ATTRS_USE_QUEUE = 'shipping/shipment_email_settings_send_track/use_mysql_queue';
    const XML_PATH_SHIPMENT_ATTRS_TURN_ON_LOGS = 'shipping/shipment_email_settings_send_track/use_logs';

    const LOG_PREFIX = "Emblz_SendTrackNumbers_Helper: ";

    /**
     * @var ProductFactory
     */
    protected $productFactory;

    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     *
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;

    /**
     * @param Context $context
     * @param ProductFactory $productFactory
     */
    public function __construct(
        Context $context,
        ProductFactory $productFactory,
        CustomerRepositoryInterface $customerRepository,
        OrderRepositoryInterface $orderRepository
    )
    {
        parent::__construct($context);
        $this->productFactory = $productFactory;
        $this->customerRepository = $customerRepository;
        $this->orderRepository = $orderRepository;
    }

    private function getMethod(Order $order)
    {
        $shippingMethod = $order->getShippingMethod();
        return is_object($shippingMethod) ? $shippingMethod['method'] : $order->getShippingMethod();
    }

    private function getCarrierCode(Order $order, Track $shipmentTrack)
    {
        $shippingMethod = $order->getShippingMethod();
        return is_object($shippingMethod) ? $shippingMethod['carrier_code'] : $shipmentTrack->getCarrierCode();
    }

    public function getConfig($config_path, $scopeCode = null)
    {

        $om = \Magento\Framework\App\ObjectManager::getInstance();
        $customerSession = $om->get('Magento\Customer\Model\Session');
        $customerData = $customerSession->getCustomer()->getData(); //get all data of customerData
        $customerData = $customerSession->getCustomer()->getId();


        return $this->scopeConfig->getValue(
            $config_path,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $scopeCode
        );
    }

    protected function getterName($a)
    {
        return "get" . str_replace(' ', '', ucwords(str_replace('_', ' ', $a)));
    }

    protected function getCustomerAttributeValues($order)
    {
        $customerId = $order->getCustomerId();
        /** @var \Magento\Sales\Model\Order $order */
        if (!$customerId) {
            return [];
        }
        $customer = $this->customerRepository->getById($order->getCustomerId());
        $attrCodesToSend = $this->getCustomerAttributesToSend($customer->getStoreId());
        if (!$attrCodesToSend) {
            return [];
        }
        $attrCodesToSendArray = explode(",", $attrCodesToSend);

        $result = [];
        /** @var \Magento\Customer\Model\Data\Customer $customer */
        foreach ($attrCodesToSendArray as $attrCode) {
            //echo get_class($customer->getStoreId());
            $methodName = $this->getterName($attrCode);
            if (method_exists($customer, $methodName)) {
                $attrVal = call_user_func([$customer, $methodName]);
            } else {
                $attrVal = $customer->getCustomAttribute($attrCode);
            }

            if (is_object($attrVal)) {
                $attrVal = (array) $attrVal;
            }

            if (!$attrVal) {
                continue;
            }

            if (is_array($attrVal)) {
                $attrVal = json_encode($attrVal, 0, 10);
            }

            $result[$attrCode] = $attrVal;
        }
        return $result;
    }

    public function getTracksData(Order $order, $tracks = null)
    {
        //error_reporting(-1);
        //ini_set("display_errors", true);

        $trackCollection = $tracks ? $tracks : $order->getTracksCollection();
        if ($trackCollection) {
            $orderNumber = $order->getRealOrderId();
            $orderDate = $order->getCreatedAt() ? substr($order->getCreatedAt(), 0, 10) : "";
            $originZIP = $this->getConfig(Shipment::XML_PATH_STORE_ZIP, $order->getStoreId());
            $shippingAddress = $order->getShippingAddress();

            $trackingNumbers = [];
            $carriers = [];
            $serviceMethods = [];
            $shippingDates = [];
            $destinationZIPs = [];
            $categoryIdsSet = [];
            $productIdsSet = [];
            $destinationCountry = "";

            /** @var \Magento\Sales\Model\Order\Shipment\Track $shipmentTrack */
            foreach ($trackCollection as $shipmentTrack) {

                $shipment = $shipmentTrack->getShipment();
                $trackingNumber = $shipmentTrack->getNumber();
                $trackingNumbers[] = $trackingNumber;

                $carrier = $this->getCarrierCode($order, $shipmentTrack);
                $carriers[] = $carrier;

                $serviceMethod = $this->getMethod($order);
                $serviceMethods[] = $serviceMethod;

                $shippingDate = $shipment->getCreatedAt() ? substr($shipment->getCreatedAt(), 0, 10) : "";
                $shippingDates[] = $shippingDate;

                $estimatedDeliveryDate = "";

                $destinationZIP = $shippingAddress ? $shippingAddress->getPostcode() : "";
                $destinationZIPs[] = $destinationZIP;

                $destinationCountry = $shippingAddress->getCountryId();
                $categoryIds = $this->getCategoryIds($shipment);
                $categoryIdsSet[] = $categoryIds;
                $productIds = $this->getProductSkus($shipment);
                $productIdsSet[] = $productIds;
                $tags = "";
            }

            $trackingNumbersStr = implode("|", array_unique($trackingNumbers));
            $carriersStr = implode("|", array_unique($carriers));
            $serviceMethodStr = implode("|", array_unique($serviceMethods));
            $shippingDatesStr = implode("|", array_unique($shippingDates));
            $destinationZIPsStr = implode("|", array_unique($destinationZIPs));
            $categoryIdsStr = implode("|", array_unique($categoryIdsSet));
            $productIdsStr = implode("|", array_unique($productIdsSet));
            if ($categoryIdsStr) $categoryMap = [ "cid" => $categoryIdsStr]; else $categoryMap = [];
            $customerAttributes = $this->getCustomerAttributeValues($order);
            $optional = $customerAttributes ? ["optional" => $customerAttributes] : [];
            return array_merge([
                "name" => $order->getCustomerFirstname() ? $order->getCustomerFirstname() : "-",
                "email" => $order->getCustomerEmail(),
                "trk" => $trackingNumbersStr,
                "on" => $orderNumber,
                "car" => $carriersStr,
                "serv" => $serviceMethodStr,
                "od" => $orderDate,
                "sd" => $shippingDatesStr,
                "ozip" => $originZIP,
                "dzip" => $destinationZIPsStr,
                "dcountry" => $destinationCountry,
                "pid" => $productIdsStr,
                "tag" => "mg"
            ], $optional, $categoryMap);
        }
        return [];
    }

    public function getTrackData(Order $order, Track $shipmentTrack)
    {
        return $this->getTracksData($order, [$shipmentTrack]);
    }

    private function getCategoryIds(Shipment $shipment)
    {
        $categoryIds = array_map(
            function ($shipmentItem) {
                $productId = $shipmentItem->getProductId();
                $product = $this->productFactory->create()->load($productId);
                return $product->getCategoryIds();
            },
            !empty($shipment->getItems()) ? $shipment->getItems() : []
        );
        $oneDimension = call_user_func_array('array_merge', $categoryIds);
        return implode(",", array_unique($oneDimension));
    }

    private function getProductSkus(Shipment $shipment)
    {
        $skus = array_map(
            function ($shipmentItem) {
                return $shipmentItem->getSku();
            },
            !empty($shipment->getItems()) ? $shipment->getItems() : []
        );
        return implode(",", array_unique($skus));
    }

    public function assembleData($track)
    {
        /** @var Track $track */
        try {
            $order = $this->orderRepository->get($track->getOrderId());
            $data = $this->getTrackData($order, $track);
            return json_encode($data);
        } catch (\Exception $e) {
            $this->log($e->getMessage() . " " . $e->getTraceAsString(), $track->getStoreId());
            return "";
        }
    }

    public function sendTrack($postData, $storeId, $trackId, $orderId)
    {
        $opts = ['http' =>
            [
                'method'  => 'POST',
                'header'  => [
                    'Accept: application/json',
                    'Content-Type: application/json',
                    'x-wismo-auth-token: ' . $this->getAuthToken($storeId)
                ],
                'content' => $postData,
                'ignore_errors' => true
            ]
        ];

        $context  = stream_context_create($opts);
        $url = $this->getUrlToSendTrack($storeId);

        //$w = stream_get_wrappers();// you might want to check if $w has "https" value in it if this code has issue
        try {
            $response = file_get_contents($url, false, $context);
            $responseCode = $http_response_header[0];
            if (str_contains($responseCode, "HTTP/1.1 20")) {
                $this->log(
                    self::LOG_PREFIX . 'track saved, trackId: ' . $trackId . ", orderId: " . $orderId . ", response: " . $response,
                    $storeId
                );
            } else {
                $this->log(
                    self::LOG_PREFIX . 'track hasn\'t been saved, trackId: ' . $trackId . ", orderId: " . $orderId .
                    ", responseCode: " . $responseCode . ", response: " . $response,
                    $storeId
                );
            }
            $this->waitIfQueue($storeId);
        } catch (\Exception $e) {
            $this->log(
                self::LOG_PREFIX . 'potential issue with server settings, trackId: ' . $trackId . ", orderId: " . $orderId . "err:" . $e->getMessage(),
                $storeId
            );
        }
    }

    public function getUrlToSendTrack($storeId)
    {
        return $this->getConfig(self::XML_PATH_SHIPMENT_SEND_TRACK_URL, $storeId);
    }

    public function getRPS($storeId)
    {
        return $this->getConfig(self::XML_PATH_SHIPMENT_RPS, $storeId);
    }

    public function getCustomerAttributesToSend($storeId)
    {
        return $this->getConfig(self::XML_PATH_SHIPMENT_ATTRS_TO_SEND, $storeId);
    }

    public function useQueue($storeId)
    {
        return $this->getConfig(self::XML_PATH_SHIPMENT_ATTRS_USE_QUEUE, $storeId);
    }

    public function areLogsEnabled($storeId)
    {
        return $this->getConfig(self::XML_PATH_SHIPMENT_ATTRS_TURN_ON_LOGS, $storeId);
    }

    public function getAuthToken($storeId)
    {
        return $this->getConfig(self::XML_PATH_SHIPMENT_AUTH_TOKEN, $storeId);
    }

    private function waitIfQueue($storeId) {
        if ($this->useQueue($storeId) && $this->getRPS($storeId)) {
            usleep(intval(1000000/$this->getRPS($storeId)));
        }
    }

    private function log($message, $storeId) {
        if ($this->areLogsEnabled($storeId)) {
            $this->_logger->info($message);
        }
    }
}
